<?php

namespace EventPortal\Domain\Util;

use EventPortal\Domain\Component\Input;
use EventPortal\Domain\Component\Section;
use EventPortal\Domain\Component\Step;
use Illuminate\Support\Collection;
use Illuminate\Validation\Rules\Unique;

class ValidationRules
{
    public Collection $rules;

    public function __construct(array $fields)
    {
        $this->rules = collect($fields)
            ->map(function ($field) {
                if ($field instanceof Step) {
                    return self::handleSteps($field);
                }

                if ($field instanceof Section) {
                    return $field->children;
                }

                return $field;
            })
            ->flatten()
            ->mapWithKeys(function (Input $input) {
                return [
                    $input->name => $input->validation,
                ];
            })
            ->filter();
    }

    public static function for(array $fields): self
    {
        return new self($fields);
    }

    public function get(): array
    {
        return $this->rules->toArray();
    }

    public function requiresReResolving(): bool
    {
        return $this
            ->rules
            ->flatten()
            ->contains(function ($rule) {
                return $rule instanceof Unique;
            });
    }

    /**
     * @param mixed $field
     * @return mixed
     */
    private static function handleSteps($field)
    {
        $containsSections = collect($field->children)
            ->filter(function ($item) {
                return $item instanceof Section;
            })
            ->isNotEmpty();

        if ($containsSections) {
            return collect($field->children)->pluck('children')->flatten();
        }

        return $field->children;
    }
}
